@@ -8,7 +8,7 @@ use HTTP::Tiny;
 use strict;
 use warnings;
 
-our $VERSION = '1.0';
+our $VERSION = '1.10';
 $VERSION = eval $VERSION;
 
 sub new {
@@ -23,12 +23,21 @@ sub new {
         'sent_to_all'   => 'false',
     };
     
-    if (exists $ENV{'GATEWAY_INTERFACE'}) {
-        read(STDIN, $vars{'payload'}, $ENV{'CONTENT_LENGTH'});
-        $vars{'webhook'} = decode_json($vars{'payload'}) if $vars{'payload'};
-        $vars{'error'}   = 'No payload data' unless $vars{'webhook'};
+    if (exists $vars{'payload'}) {
+        $vars{'webhook'} = eval { decode_json($vars{'payload'});};
+        $vars{'error'}   = 'Missing payload data' unless $vars{'webhook'};
     } else {
-        $vars{'error'}   = 'Looks like this is not a web request!';
+        # Obtaining  payload from STDIN only 
+        # exists for backward  compatability
+        # This option is deprecated and will
+        # be  removed  in  a  future version
+        if (exists $ENV{'GATEWAY_INTERFACE'}) {
+            read(STDIN, $vars{'payload'}, $ENV{'CONTENT_LENGTH'});
+            $vars{'webhook'} = decode_json($vars{'payload'}) if $vars{'payload'};
+            $vars{'error'}   = 'No payload data' unless $vars{'webhook'};
+        } else {
+            $vars{'error'}   = 'Looks like this is not a web request!';
+        }
     }
     
     return bless \%vars, $class;
@@ -108,7 +117,8 @@ sub check_signature {
     my $signed_payload = $sig_head{'t'} . '.' . $self->{'payload'};
     
     if (!defined $sig_head{'v1'}) {
-        die "No v1";
+        $self->_error("No v1");
+        return undef;
     }
     
     if (hmac_sha256_hex($signed_payload, $self->{'signing_secret'}) eq $sig_head{'v1'}) {
@@ -203,18 +213,24 @@ L<Business::Stripe::Webhook> - A Perl module for handling webhooks sent by Strip
 
 =head1 VERSION
 
-Version 1.0
+Version 1.10
 
 =head1 SYNOPSIS
 
   use Stripe::Webhook;
+  
+  my $payload;
+  read(STDIN, $payload, $ENV{'CONTENT_LENGTH'});
 
   my $webhook = Stripe::Webhook->new(
       signing_secret                => 'whsec_...',
       api_secret                    => 'sk_test_...',
+      payload                       => $payload,
       invoice-paid                  => \&update_invoice,
       checkout-session-completed    => \&update_session,
   );
+  
+  die $webhook->error unless $webhook->success;
 
   my $result = $webhook->process();
   
@@ -270,6 +286,7 @@ Stripe requires a timely reply to webhook calls.  Therefore, if you need to carr
   
   my $webhook = Stripe::Webhook->new(
       signing_secret    => 'whsec_...',
+      payload           => $payload,
       invoice-paid      => \&update_invoice,
   );
 
@@ -309,6 +326,7 @@ Creates a new Stripe::Webhook object.
 
   my $webhook = Stripe::Webhook->new(
       signing_secret => 'whsec_...',
+      payload        => $payload,
   );
 
 This method takes one or more parameters:
@@ -321,6 +339,10 @@ B<signing_secret>: The webhook signing secret provided by Stripe. If omitted, th
 
 =item *
 
+B<payload>: A JSON string.  Required (I<see below>). The JSON object from Stripe.
+
+=item *
+
 B<api_secret>: The Stripe secret API Key - see L<https://stripe.com/docs/keys>. Optional but will be required if the C<get_subscription> method is needed.
 
 =item *
@@ -347,6 +369,8 @@ B<warning>: A callback subroutine to handle warnings.  If not defined, warnings
 
 =back
 
+Previous versions on L<Business::Stripe::Webhook> allowed the B<payload> parameter to be omitted.  In this case, the module would read C<STDIN> to obtain the JSON string.  This continues to work for backward compatability only but will be removed from furture versions.
+
 =head2 success
 
 Returns true if the last operation was successful, or false otherwise.
@@ -447,8 +471,12 @@ Here's an example of how to use the module to handle a webhook:
 
   use Business::Stripe::Webhook;
   
+  my $payload;
+  read(STDIN, $payload, $ENV{'CONTENT_LENGTH'});
+  
   my $webhook = Business::Stripe::Webhook->new(
       signing_secret => 'whsec_...',
+      payload        => $payload,
       invoice-paid   => \&pay_invoice,
   );
   
@@ -484,10 +512,10 @@ Here's an example of how to use the module to retrieve a subscription object:
 =head1 SEE ALSO
 
 L<Stripe Subscriptions API|https://stripe.com/docs/api/subscriptions>
- 
-L<Business::Stripe::WebCheckout>
 
 L<Business::Stripe::Subscription>
+ 
+L<Business::Stripe::WebCheckout>
 
 =head1 AUTHOR
 
@@ -495,15 +523,15 @@ Ian Boddison <ian at boddison.com>
 
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-business-stripe-subscription at rt.cpan.org>, or through
-the web interface at L<https://rt.cpan.org/NoAuth/ReportBug.html?Queue=bug-business-stripe-subscription>.  I will be notified, and then you'll
+Please report any bugs or feature requests to C<bug-business-stripe-webhook at rt.cpan.org>, or through
+the web interface at L<https://rt.cpan.org/NoAuth/ReportBug.html?Queue=bug-business-stripe-webhook>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
 
-    perldoc Business::Stripe::Subscription
+    perldoc Business::Stripe::Webhook
 
 You can also look for information at:
 
@@ -511,11 +539,11 @@ You can also look for information at:
 
 =item * RT: CPAN's request tracker (report bugs here)
 
-L<https://rt.cpan.org/NoAuth/Bugs.html?Dist=Business-Stripe-Subscription>
+L<https://rt.cpan.org/NoAuth/Bugs.html?Dist=Business-Stripe-Webhook>
 
 =item * Search CPAN
 
-L<https://metacpan.org/release/Business::Stripe::Subscription>
+L<https://metacpan.org/release/Business::Stripe::Webhook>
 
 =back
 
